@@ -78,16 +78,21 @@ let rec trm_to_coq c m bound lp rp =
 	end
   | Ap(Imp,m1) -> trm_to_coq c (Lam(Prop,Ap(Ap(Imp,shift m1 0 1),DB(0,Prop)))) bound lp rp;
   | Imp -> trm_to_coq c (Lam(Prop,Lam(Prop,Ap(Ap(Imp,DB(1,Prop)),DB(0,Prop))))) bound lp rp; 
-  | Ap(Forall(a),m1) -> (* forall *)
-	begin
+  | Ap(Forall(a),Lam(_,m1)) -> (* forall with Lam *)
+      begin
 	if ((lp >= 0) || (rp >= 0)) then Printf.fprintf c "(";
 	begin
 	  Printf.fprintf c "forall";
 	  print_all_coq c a m1 bound
 	end;
 	if ((lp >= 0) || (rp >= 0)) then Printf.fprintf c ")";
-	end
-  | Forall(_) -> Printf.fprintf c "(fun p => forall x, p x)"
+      end
+  | Forall(a) ->
+      begin
+	if ((lp >= 5000) || (rp >= 5001)) then Printf.fprintf c "(";
+	Printf.fprintf c "SPi "; print_stp_coq c a coq_names true;
+	if ((lp >= 5000) || (rp >= 5001)) then Printf.fprintf c ")";
+      end
   | Ap(Ap(Eq(a),m1),m2) -> (* Equality *)
       if ((lp < 40) && (rp < 40)) then
 	begin
@@ -103,43 +108,45 @@ let rec trm_to_coq c m bound lp rp =
 	  trm_to_coq c m2 bound 40 (-1);
 	  Printf.fprintf c ")";
 	end
-  | Ap(Eq(a),m) ->    
-	if ((lp < 5000) && (rp < 5001)) then
-	begin
-	  Printf.fprintf c "eq ";
-	  trm_to_coq c m bound 5001 rp;
-	end
-      else
-	begin
-	  Printf.fprintf c "(eq ";
-	  trm_to_coq c m bound 5001 (-1);
-	  Printf.fprintf c ")";
-	end  
   | Eq(a) ->     
 	if ((lp < 5000) && (rp < 5001)) then
 	begin
-	  Printf.fprintf c "@eq ";
+	  Printf.fprintf c "Seq ";
 	  print_stp_coq c a coq_names true;
 	end
       else
 	begin
-	  Printf.fprintf c "(@eq ";
+	  Printf.fprintf c "(Seq ";
 	  print_stp_coq c a coq_names true;
 	  Printf.fprintf c ")";
 	end      
+(*** I'm now always explicitly giving the Stype
   | Ap(Choice(a),m) ->   (* Choice *)  
 	if ((lp < 5000) && (rp < 5001)) then
 	begin
-	  Printf.fprintf c "Sepsilon ";
+	  Printf.fprintf c "@Sepsilon ";
 	  trm_to_coq c m bound 5001 rp;
 	end
       else
 	begin
-	  Printf.fprintf c "(Sepsilon ";
+	  Printf.fprintf c "(@Sepsilon ";
+	  
 	  trm_to_coq c m bound 5001 (-1);
 	  Printf.fprintf c ")";
 	end      
-  | Choice(a) -> Printf.fprintf c "@Sepsilon "; print_stp_coq c a coq_names true;
+***)
+  | Choice(a) ->
+      if ((lp < 5000) && (rp < 5001)) then
+	begin
+	  Printf.fprintf c "@Sepsilon ";
+	  print_stp_coq c a coq_names true
+	end
+      else
+	begin
+	  Printf.fprintf c "(@Sepsilon ";
+	  print_stp_coq c a coq_names true;
+	  Printf.fprintf c ")"
+	end
   | True -> (* Top *)
 	Printf.fprintf c "True"
   | Ap(Ap(And,m1),m2) -> (* conjunction *)
@@ -191,7 +198,7 @@ let rec trm_to_coq c m bound lp rp =
 	end
   | Iff -> Printf.fprintf c "iff"
   | Neg -> Printf.fprintf c "not"
-  | Ap(Exists(a),m1) -> (* exist *)
+  | Ap(Exists(a),Lam(_,m1)) -> (* exist *)
       begin
 	if ((lp >= 0) || (rp >= 0)) then Printf.fprintf c "(";
 	begin
@@ -199,7 +206,12 @@ let rec trm_to_coq c m bound lp rp =
 	end;
 	if ((lp >= 0) || (rp >= 0)) then Printf.fprintf c ")";
       end
-  | Exists(_) ->  Printf.fprintf c "(fun p => exists x, p x)" 
+  | Exists(a) ->
+      begin
+	if ((lp >= 5000) || (rp >= 5001)) then Printf.fprintf c "(";
+	Printf.fprintf c "SSigma "; print_stp_coq c a coq_names true;
+	if ((lp >= 5000) || (rp >= 5001)) then Printf.fprintf c ")";
+      end
   | DB(i,a) -> (* Bound variable *)
 	Printf.fprintf c "%s" (Variables.get i bound)
   | Lam(a,m) ->
@@ -238,12 +250,11 @@ and print_lam_coq c a m bound =
 
 (* Prints consecutive forall-terms together with the corresponding lambda-terms as a single forall in Coq. *) 		
 and print_all_coq c a m bound =
-	let bound = Variables.push bound in
-	Printf.fprintf c " ("; Printf.fprintf c "%s" (Variables.top bound); Printf.fprintf c ":"; print_stp_coq c a coq_names false; Printf.fprintf c ")";
-	match m with
-		| Lam(_,Ap(Forall(a'),m'))-> print_all_coq c a' m' bound
-		| Lam(_,m') -> Printf.fprintf c ", "; trm_to_coq c m' bound (-1) (-1)
-		| _ -> Printf.fprintf c ", "; trm_to_coq c (Ap(shift m 0 1,DB(0,a))) bound (-1) (-1)
+  let bound = Variables.push bound in
+  Printf.fprintf c " ("; Printf.fprintf c "%s" (Variables.top bound); Printf.fprintf c ":"; print_stp_coq c a coq_names false; Printf.fprintf c ")";
+  match m with
+  | Ap(Forall(a'),Lam(_,m'))-> print_all_coq c a' m' bound
+  | _ -> Printf.fprintf c ", "; trm_to_coq c m bound (-1) (-1)
 
 (* Prints an exist-term together with the corresponding lambda-term as an exists in Coq. *) 		
 and print_ex_coq c a m bound =
@@ -251,23 +262,20 @@ and print_ex_coq c a m bound =
 	Printf.fprintf c "exists"; Printf.fprintf c " %s" (Variables.top bound); 
 	Printf.fprintf c ":"; print_stp_coq c a coq_names false; 
         Printf.fprintf c ", ";
-	match m with
-		| Lam(_,m') -> trm_to_coq c m' bound (-1) (-1)
-		| _ -> trm_to_coq c (Ap(shift m 0 1,DB(0,a))) bound (-1) (-1)
-
+	trm_to_coq c m bound (-1) (-1)
 
 (** Input: Name x, Type a, association list (term -> hypothesis name) hyp 
 	Output: name of the hypothesis that defines x as a choice operator **)
 let get_Choicop_axiom x a hyp = 
 let ao = Ar(a,Prop) in
-let m1 = Ap (Forall (Ar (a, Prop)),Lam (Ar (a, Prop),Ap (Forall a,Lam (a,Ap (Ap (Imp, Ap (DB (1, Ar (a, Prop)), DB (0, a))),
-	 Ap (DB (1, Ar (a, Prop)),Ap (Name (x, Ar (Ar (a, Prop), a)), DB (1, Ar (a, Prop))))))))) in
-let m2 = Ap (Forall (Ar (a, Prop)),Lam (Ar (a, Prop),Ap(Ap (Imp,Ap(Ap (Imp,Ap (Forall a,Lam (a,
-	Ap (Ap (Imp, Ap (DB (1, Ar (a, Prop)), DB (0, a))),False)))),False)),Ap (DB (0, Ar (a, Prop)),
-	Ap (Name (x, Ar (Ar (a, Prop), a)),DB (0, Ar (a, Prop)))))))in
-let m3 = Ap (Forall (Ar (a, Prop)),Lam (Ar (a, Prop),Ap(Ap (Imp,Ap (Exists a,Lam (a,
-	Ap (DB (1, Ar (a, Prop)), DB (0, a))))),Ap (DB (0, Ar (a, Prop)),
-	Ap (Name (x, Ar (Ar (a, Prop), a)),DB (0, Ar (a, Prop)))))))in
+let m1 = Ap (Forall (ao),Lam (ao,Ap (Forall a,Lam (a,Ap (Ap (Imp, Ap (DB (1, ao), DB (0, a))),
+	 Ap (DB (1, ao),Ap (Name (x, Ar (ao, a)), DB (1, ao)))))))) in
+let m2 = Ap (Forall (ao),Lam (ao,Ap(Ap (Imp,Ap(Ap (Imp,Ap (Forall a,Lam (a,
+	Ap (Ap (Imp, Ap (DB (1, ao), DB (0, a))),False)))),False)),Ap (DB (0, ao),
+	Ap (Name (x, Ar (ao, a)),DB (0, ao))))))in
+let m3 = Ap (Forall (ao),Lam (ao,Ap(Ap (Imp,Ap (Exists a,Lam (a,
+	Ap (DB (1, ao), DB (0, a))))),Ap (DB (0, ao),
+	Ap (Name (x, Ar (ao, a)),DB (0, ao))))))in
 try
 let (m,h)= List.find (fun (m,h) -> m = m1 || m = m2 || m = m3 ) hyp in h
 with Not_found -> "missing_choice_axiom_for"^x
@@ -299,139 +307,155 @@ List.fold_left (fun cons (x,a) ->
 	if List.mem (x,a) cons then cons 
 	else (Printf.fprintf c "%stab_inh (" sp; print_stp_coq c a coq_names false; Printf.fprintf c ") %s.\n" x ;(x,a)::cons) ) 
 	const (find_fresh_const (coqnorm n))
+
+let rec lookup w s hyp =
+  try
+    List.assoc s hyp
+  with
+  | Not_found ->
+      Printf.printf "%s: Could not find Coq hyp name\ns = %s\nhyp:\n" w (trm_str s);
+      List.iter (fun (m,h) -> Printf.printf "%s: %s\n" h (trm_str m)) hyp;
+      failwith ("Could not find Coq hyp name")
  
 (** Input: out_channel c, refutation r, association list (term -> hypothesis name) hyp, association list (constant name -> type) const, Space string sp 
 	Output: unit, prints refutation r to c **)
 let rec ref_coq1 c r hyp const sp=
 	match r with
  | Conflict(s,ns) -> 			
-	Printf.fprintf c "%stab_conflict %s %s.\n" sp (List.assoc (coqnorm s) hyp) (List.assoc (coqnorm ns) hyp)
+	Printf.fprintf c "%stab_conflict %s %s.\n" sp (lookup "0" (coqnorm s) hyp) (lookup "1" (coqnorm ns) hyp)
  | Fal(_) -> 				
-	Printf.fprintf c "%stab_false %s.\n" sp (List.assoc False hyp) 
+	Printf.fprintf c "%stab_false %s.\n" sp (lookup "2" False hyp) 
  | NegRefl(s) -> 			
-	Printf.fprintf c "%stab_refl %s.\n" sp (List.assoc (coqnorm s) hyp)
+	Printf.fprintf c "%stab_refl %s.\n" sp (lookup "3" (coqnorm s) hyp)
  | Implication(h,s,t,r1,r2) -> 	
 	let h1 = get_hyp_name() in	
-	Printf.fprintf c "%stab_imp %s %s.\n" sp (List.assoc (coqnorm h) hyp) h1;
+	Printf.fprintf c "%stab_imp %s %s.\n" sp (lookup "4" (coqnorm h) hyp) h1;
 	ref_coq1 c r1 ((coqnorm s,h1)::hyp) const (sp^" ");
 	ref_coq1 c r2 ((coqnorm t,h1)::hyp) const (sp^" ");
  | Disjunction(h,s,t,r1,r2) ->
 	let h1 = get_hyp_name() in	
-	Printf.fprintf c "%stab_or %s %s.\n" sp (List.assoc (coqnorm h) hyp) h1;
+	Printf.fprintf c "%stab_or %s %s.\n" sp (lookup "5" (coqnorm h) hyp) h1;
 	ref_coq1 c r1 ((coqnorm s,h1)::hyp) const (sp^" ");
 	ref_coq1 c r2 ((coqnorm t,h1)::hyp) const (sp^" "); 	
  | NegConjunction(h,s,t,r1,r2) ->
 	let h1 = get_hyp_name() in	
-	Printf.fprintf c "%stab_nand %s %s.\n" sp (List.assoc (coqnorm h) hyp) h1;
+	Printf.fprintf c "%stab_nand %s %s.\n" sp (lookup "6" (coqnorm h) hyp) h1;
 	ref_coq1 c r1 ((coqnorm s,h1)::hyp) const (sp^" ");
 	ref_coq1 c r2 ((coqnorm t,h1)::hyp) const (sp^" ");  
  | NegImplication(h,s,t,r1) ->
 	let h1 = get_hyp_name() in
 	let h2 = get_hyp_name() in	
-	Printf.fprintf c "%stab_negimp %s %s %s.\n" sp (List.assoc (coqnorm h) hyp) h1 h2;
+	Printf.fprintf c "%stab_negimp %s %s %s.\n" sp (lookup "7" (coqnorm h) hyp) h1 h2;
 	ref_coq1 c r1 ((coqnorm s,h1)::(coqnorm t,h2)::hyp) const sp
  | Conjunction(h,s,t,r1) ->
 	let h1 = get_hyp_name() in
 	let h2 = get_hyp_name() in	
-	Printf.fprintf c "%stab_and %s %s %s.\n" sp (List.assoc (coqnorm h) hyp) h1 h2;
+	Printf.fprintf c "%stab_and %s %s %s.\n" sp (lookup "8" (coqnorm h) hyp) h1 h2;
 	ref_coq1 c r1 ((coqnorm s,h1)::(coqnorm t,h2)::hyp) const sp
  | NegDisjunction(h,s,t,r1) ->
 	let h1 = get_hyp_name() in
 	let h2 = get_hyp_name() in	
-	Printf.fprintf c "%stab_nor %s %s %s.\n" sp (List.assoc (coqnorm h) hyp) h1 h2;
+	Printf.fprintf c "%stab_nor %s %s %s.\n" sp (lookup "9" (coqnorm h) hyp) h1 h2;
 	ref_coq1 c r1 ((coqnorm s,h1)::(coqnorm t,h2)::hyp) const sp
  | All(h,s,r1,a,m,n) ->
 	let const = add_fresh_const c const n sp in
 	let h1 = get_hyp_name() in	
-	Printf.fprintf c "%stab_all %s (" sp (List.assoc (coqnorm h) hyp); 
+	Printf.fprintf c "%stab_all %s (" sp (lookup "10" (coqnorm h) hyp); 
 	(trm_to_coq c n (Variables.make ()) (-1) (-1));
 	Printf.fprintf c ") %s.\n" h1;
 	ref_coq1 c r1 ((coqnorm s,h1)::hyp) const sp
  | NegAll(h,s,r1,a,m,x) ->
 	let h1 = get_hyp_name() in
 	let x = ( Hashtbl.find coq_names x ) in
-	Printf.fprintf c "%stab_negall %s %s %s.\n" sp (List.assoc (coqnorm h) hyp) x h1;
+	Printf.fprintf c "%stab_negall %s %s %s.\n" sp (lookup "11" (coqnorm h) hyp) x h1;
 	ref_coq1 c r1 ((coqnorm s,h1)::hyp) ((x,a)::const) sp
  | Exist(h,s,r1,a,m,x) ->
 	let h1 = get_hyp_name() in
 	let x = ( Hashtbl.find coq_names x ) in
-	Printf.fprintf c "%stab_ex %s %s %s.\n" sp (List.assoc (coqnorm h) hyp) x h1;
+	Printf.fprintf c "%stab_ex %s %s %s.\n" sp (lookup "12" (coqnorm h) hyp) x h1;
 	ref_coq1 c r1 ((coqnorm s,h1)::hyp) ((x,a)::const) sp
  | NegExist(h,s,r1,a,m,n) ->
 	let const = add_fresh_const c const n sp in
 	let h1 = get_hyp_name() in	
-	Printf.fprintf c "%stab_negex %s (" sp (List.assoc (coqnorm h) hyp); 
+	Printf.fprintf c "%stab_negex %s (" sp (lookup "13" (coqnorm h) hyp); 
 	(trm_to_coq c n (Variables.make ()) (-1) (-1));
 	Printf.fprintf c ") %s.\n" h1;
 	ref_coq1 c r1 ((coqnorm s,h1)::hyp) const sp	
  | Mating(h1,h2, ss, rs) ->
 	let h3 = get_hyp_name() in	
-	Printf.fprintf c "%stab_mat %s %s %s.\n" sp (List.assoc (coqnorm h1) hyp) (List.assoc (coqnorm h2) hyp) h3;
+	Printf.fprintf c "%stab_mat %s %s %s.\n" sp (lookup "14" (coqnorm h1) hyp) (lookup "15" (coqnorm h2) hyp) h3;
 	List.iter (fun (s,r) -> ref_coq1 c r ((coqnorm s,h3)::hyp) const (sp^" ")) (List.combine ss rs)
  | Decomposition(h1, ss, rs) ->
 	let h3 = get_hyp_name() in	
-	Printf.fprintf c "%stab_dec %s %s.\n" sp (List.assoc (coqnorm h1) hyp) h3;
+	Printf.fprintf c "%stab_dec %s %s.\n" sp (lookup "16" (coqnorm h1) hyp) h3;
 	List.iter (fun (s,r) -> ref_coq1 c r ((coqnorm s,h3)::hyp) const (sp^" ")) (List.combine ss rs) 	
  | Confront(h1,h2,su,tu,sv,tv,r1,r2) ->
 	let h3 = get_hyp_name() in
 	let h4 = get_hyp_name() in	
-	Printf.fprintf c "%stab_con %s %s %s %s.\n" sp (List.assoc (coqnorm h1) hyp) (List.assoc (coqnorm h2) hyp) h3 h4;
+	Printf.fprintf c "%stab_con %s %s %s %s.\n" sp (lookup "17" (coqnorm h1) hyp) (lookup "18" (coqnorm h2) hyp) h3 h4;
 	ref_coq1 c r1 ((coqnorm su,h3)::(coqnorm tu,h4)::hyp) const (sp^" ");
 	ref_coq1 c r2 ((coqnorm sv,h3)::(coqnorm tv,h4)::hyp) const (sp^" ");	
  | Trans(h1,h2,su,r1) ->
 	let h3 = get_hyp_name() in	
-	Printf.fprintf c "%stab_trans %s %s %s.\n" sp (List.assoc (coqnorm h1) hyp) (List.assoc (coqnorm h2) hyp) h3;
+	Printf.fprintf c "%stab_trans %s %s %s.\n" sp (lookup "19" (coqnorm h1) hyp) (lookup "20" (coqnorm h2) hyp) h3;
 	ref_coq1 c r1 ((coqnorm su,h3)::hyp) const (sp^" ");
  | NegEqualProp(h,s,t,r1,r2) -> 
 	let h1 = get_hyp_name() in
 	let h2 = get_hyp_name() in	
-	Printf.fprintf c "%stab_be %s %s %s.\n" sp (List.assoc (coqnorm h) hyp) h1 h2;
+	Printf.fprintf c "%stab_be %s %s %s.\n" sp (lookup "21" (coqnorm h) hyp) h1 h2;
 	ref_coq1 c r1 ((coqnorm s,h1)::(coqnorm (neg t),h2)::hyp) const (sp^" ");
 	ref_coq1 c r2 ((coqnorm (neg s),h1)::(coqnorm t,h2)::hyp) const (sp^" ");
  | EqualProp(h,s,t,r1,r2) -> 
 	let h1 = get_hyp_name() in
 	let h2 = get_hyp_name() in	
-	Printf.fprintf c "%stab_bq %s %s %s.\n" sp (List.assoc (coqnorm h) hyp) h1 h2;
+	Printf.fprintf c "%stab_bq %s %s %s.\n" sp (lookup "22" (coqnorm h) hyp) h1 h2;
 	ref_coq1 c r1 ((coqnorm s,h1)::(coqnorm t,h2)::hyp) const (sp^" ");
 	ref_coq1 c r2 ((coqnorm (neg s),h1)::(coqnorm (neg t),h2)::hyp) const (sp^" ");
  | NegAequivalenz(h,s,t,r1,r2) -> 
 	let h1 = get_hyp_name() in
 	let h2 = get_hyp_name() in	
-	Printf.fprintf c "%stab_negiff %s %s %s.\n" sp (List.assoc (coqnorm h) hyp) h1 h2;
+	Printf.fprintf c "%stab_negiff %s %s %s.\n" sp (lookup "23" (coqnorm h) hyp) h1 h2;
 	ref_coq1 c r1 ((coqnorm s,h1)::(coqnorm (neg t),h2)::hyp) const (sp^" ");
 	ref_coq1 c r2 ((coqnorm (neg s),h1)::(coqnorm t,h2)::hyp) const (sp^" ");
  | Aequivalenz(h,s,t,r1,r2) -> 
 	let h1 = get_hyp_name() in
 	let h2 = get_hyp_name() in	
-	Printf.fprintf c "%stab_iff %s %s %s.\n" sp (List.assoc (coqnorm h) hyp) h1 h2;
+	Printf.fprintf c "%stab_iff %s %s %s.\n" sp (lookup "24" (coqnorm h) hyp) h1 h2;
 	ref_coq1 c r1 ((coqnorm s,h1)::(coqnorm t,h2)::hyp) const (sp^" ");
 	ref_coq1 c r2 ((coqnorm (neg s),h1)::(coqnorm (neg t),h2)::hyp) const (sp^" ");
  | NegEqualFunc(h,s,r1) ->
 	let h1 = get_hyp_name() in	
-	Printf.fprintf c "%stab_fe %s %s.\n" sp (List.assoc (coqnorm h) hyp) h1;
+	Printf.fprintf c "%stab_fe %s %s.\n" sp (lookup "25" (coqnorm h) hyp) h1;
 	ref_coq1 c r1 ((coqnorm s,h1)::hyp) const sp
  | EqualFunc(h,s,r1) ->
 	let h1 = get_hyp_name() in	
-	Printf.fprintf c "%stab_fq %s %s.\n" sp (List.assoc (coqnorm h) hyp) h1;
+	Printf.fprintf c "%stab_fq %s %s.\n" sp (lookup "26" (coqnorm h) hyp) h1;
 	ref_coq1 c r1 ((coqnorm s,h1)::hyp) const  sp
  | ChoiceR(eps,pred,s,t,r1,r2) -> 
-	let const = add_fresh_const c const pred sp in
-	let h1 = get_hyp_name() in	begin
-	match eps with
-	| Choice(a) -> 
-	Printf.fprintf c "%stab_choice (" sp;
-	(trm_to_coq c pred (Variables.make ()) (-1) (-1));
-	Printf.fprintf c ") %s.\n" h1;
-	ref_coq1 c r1 ((coqnorm s,h1)::hyp) const (sp^" ");
-	ref_coq1 c r2 ((coqnorm t,h1)::hyp) const (sp^" ");
-	| Name(x,Ar(Ar(a,Prop),_)) ->
-	Printf.fprintf c "%stab_choice' (" sp;
-	(trm_to_coq c pred (Variables.make ()) (-1) (-1));
-	Printf.fprintf c ") %s %s.\n" (get_Choicop_axiom x a hyp) h1;
-	ref_coq1 c r1 ((coqnorm s,h1)::hyp) const (sp^" ");
-	ref_coq1 c r2 ((coqnorm t,h1)::hyp) const (sp^" ");
-	| _ -> failwith "eps is not a valid epsilon"
-	end
+     let const = add_fresh_const c const pred sp in
+     let h1 = get_hyp_name() in
+     begin
+       match eps with
+       | Choice(a) -> 
+	   Printf.fprintf c "%stab_choice " sp;
+	   print_stp_coq c a coq_names true;
+	   Printf.fprintf c " (";
+	   (trm_to_coq c pred (Variables.make ()) (-1) (-1));
+	   Printf.fprintf c ") %s.\n" h1;
+	   ref_coq1 c r1 ((coqnorm s,h1)::hyp) const (sp^" ");
+	   ref_coq1 c r2 ((coqnorm t,h1)::hyp) const (sp^" ");
+       | Name(x,Ar(Ar(a,Prop),_)) ->
+	   Printf.fprintf c "%stab_choice' " sp;
+	   print_stp_coq c a coq_names true;
+	   Printf.fprintf c " (";
+	   (trm_to_coq c eps (Variables.make ()) (-1) (-1));
+	   Printf.fprintf c ") (";
+	   (trm_to_coq c pred (Variables.make ()) (-1) (-1));
+	   Printf.fprintf c ") %s %s.\n" (get_Choicop_axiom x a hyp) h1;
+	   ref_coq1 c r1 ((coqnorm s,h1)::hyp) const (sp^" ");
+	   ref_coq1 c r2 ((coqnorm t,h1)::hyp) const (sp^" ");
+       | _ -> failwith "eps is not a valid epsilon"
+     end
  | Cut(s,r1,r2) -> 
 	let const = add_fresh_const c const s sp in
 	let h1 = get_hyp_name() in	
@@ -442,11 +466,11 @@ let rec ref_coq1 c r hyp const sp=
 	ref_coq1 c r1 ((coqnorm s,h1)::hyp) const (sp^" ");
  | DoubleNegation(h,s,r1) ->
 	let h1 = get_hyp_name() in	
-	Printf.fprintf c "%stab_dn %s %s.\n" sp (List.assoc (coqnorm h) hyp) h1;
+	Printf.fprintf c "%stab_dn %s %s.\n" sp (lookup "27" (coqnorm h) hyp) h1;
 	ref_coq1 c r1 ((coqnorm s,h1)::hyp) const sp;
  | Rewrite(prefix,pt,pt',r1) ->
 	let h =  coqnorm (Ap(prefix,pt)) in
-	let h1 = List.assoc h hyp in	
+	let h1 = lookup "28" h hyp in	
 	let s =  coqnorm (Ap(prefix,pt')) in 
 	let h2 = get_hyp_name() in
 	begin
@@ -485,9 +509,25 @@ let rec ref_coq1 c r hyp const sp=
 	end;
 	ref_coq1 c r1 ((s,h2)::hyp) const sp
  | Delta(h,s,x,r1) ->
-	let h1 = (List.assoc (coqnorm h) hyp) in	
+	let h1 = (lookup "29" (coqnorm h) hyp) in	
 	Printf.fprintf c "%sunfold %s in %s.\n" sp ( Hashtbl.find coq_names x ) h1;
 	ref_coq1 c r1 ((coqnorm s,h1)::hyp) const sp;
+ | KnownResult(s,name,al,r1) ->
+     begin
+       match al with
+       | (_::_) ->
+	   let h1 = get_hyp_name() in
+	   Printf.fprintf c "%sset (%s := (%s" sp h1 name;
+	   List.iter
+	     (fun a ->
+	       Printf.fprintf c " ";
+	       print_stp_coq c a coq_names true)
+	     al;
+	   Printf.fprintf c ")).\n";
+	   ref_coq1 c r1 ((coqnorm s,h1)::hyp) const sp
+       | [] ->
+	   ref_coq1 c r1 ((coqnorm s,name)::hyp) const sp
+     end;
  | NYI(h,s,r1) -> failwith("NYI step found in ref_coq" )
  | Timeout -> failwith("Timeout step found in ref_coq" )
  | _ -> failwith("unknown refutation case in ref_coq" )    
@@ -495,10 +535,11 @@ let rec ref_coq1 c r hyp const sp=
  (** Prints refutation r to out_channel c **)
 let ref_coq c r = 
 	(* get conjecture *)
-	let con =match !conjecture with Some(_,con)->coqnorm con | None-> False in
+	let con =match !conjecture with Some(_,con,_)->coqnorm con | None-> False in
 	(* initialise hypotheses *)
 	let hyp = List.fold_left (fun l (s,pt) -> (coqnorm pt,s)::l ) [] !coqsig_hyp_trm in
 	let h1 = get_hyp_name() in
   Printf.fprintf c "\ntab_start %s.\n" h1;
   ref_coq1 c r ((neg con,h1)::hyp) (!coqsig_const) ""; 
-  Printf.fprintf c "Qed.\n" 
+  Printf.fprintf c "Qed.\n";
+  Printf.fprintf c "End SatallaxProblem.\n" 
