@@ -29,5 +29,11 @@ type refutation =
  | DoubleNegation of trm * trm * refutation  
  | Rewrite of trm * trm * trm * refutation  
  | Delta of trm * trm * string * refutation
+ | KnownResult of trm * string * stp list * refutation (* Chad: Aug 2011 *)
  | NYI of trm * trm * refutation   
  | Timeout 
+
+val ref_str : refutation -> string
+
+val is_an_eqn_p : trm -> int -> trm * trm * trm * stp
+
