@@ -307,7 +307,7 @@ let strategy_schedule_8 = [
 ];;
 
 let print_help () =
-  (print_string "Usage: satallax [-[Vvp]] [-verbose <int>] [-P <PicoMus>] [-M <modedir>] [-m <modefile>] [-t <timeout in seconds>] <problemfile>";
+  (print_string "Usage: satallax [-[Vvp]] [-verb <int>] [-P <PicoMus>] [-M <modedir>] [-m <modefile>] [-t <timeout in seconds>] <problemfile>";
    print_newline();
    print_string "-M <dir> : Set the directory in which the mode file is stored.";
    print_newline();
@@ -321,11 +321,11 @@ let print_help () =
    print_newline();
    print_string "-v : Verbose";
    print_newline();
-   print_string "-verbose <int> : Verbosity of given level";
+   print_string "-verb <int> : Verbosity of given level, -verb 0 means silent";
    print_newline();
-   print_string "-c [<file>|-] : Create a Coq version of problem, with the proof term if -p is included and proof search succeeds";
+   print_string "-c [<file>|-] : Create a Coq version of problem, with a proof script if -p is included and proof search succeeds";
    print_newline();
-   print_string "-p : Print proof term in Coq notation";
+   print_string "-p [coqscript|hocore]: Output a kind of proof object";
    print_newline());;
 
 (*** Load the mode (error if cannot find it) ***)
@@ -448,9 +448,21 @@ try
 	    else
 	      begin
 		coqfile := cf;
-		coqchannel := Some (open_out cf)
+		coqchannel := Some (open_out cf);
 	      end
 	  end
+      | "-p" ->
+	  begin
+	    incr i;
+	    let pfkind = Sys.argv.(!i) in
+	    slaveargs := (pfkind::"-p"::!slaveargs);
+	    if ((String.lowercase pfkind) = "coqscript") then
+	      mkproofterm := Some CoqScript
+	    else if ((String.lowercase pfkind) = "hocore") then
+	      mkproofterm := Some HOCore
+	    else
+	      raise (InputHelpError ("Unknown kind of proof " ^ pfkind ^ " for -p"))
+	  end
       | "-verb" ->
 	  begin
 	    incr i;
@@ -465,7 +477,6 @@ try
               match option.[j] with
               | 'v' -> slaveargs := ("-v"::!slaveargs); verbosity := 5
               | 'V' -> Version.print_version() ; print_newline(); exit 0
-              | 'p' -> slaveargs := ("-p"::!slaveargs); mkproofterm := true
               | _ -> raise (InputHelpError (String.concat " " ["Unknown command line argument";String.make 1 (option.[j])]))
             done
 	  else
@@ -479,18 +490,18 @@ try
 	begin
 	  try
 	    ignore (List.map load_mode (!mode));
-	    if (!mkproofterm) then
-	      begin
-		if ((get_bool_flag "ENABLE_PATTERN_CLAUSES") || (get_bool_flag "PATTERN_CLAUSES_TRANSITIVITY_EQ")) then
-		  begin (*** Proof terms are not yet supported for pattern clauses - June 2011 ***)
-		    raise (GenericError ("Proof terms are not supported for given mode yet"))
-		  end;
-		if (not (!coq)) then
-		  begin (*** -p implies -c - if -c was not given. - Chad, June 2011 ***)
-		    coq := true;
-		    coqchannel := Some stdout;
-		  end;
-	      end;
+	    begin
+	      match (!mkproofterm) with
+	      | Some CoqScript ->
+		  begin
+		    if (not (!coq)) then
+		      begin (*** -p implies -c - if -c was not given. - Chad, June 2011 ***)
+			coq := true;
+			coqchannel := Some stdout;
+		      end;
+		  end
+	      | _ -> ()
+	    end;
 	    if (!verbosity > 8) then print_flags () else ();
 	    let s = (get_timeout_default 0.0) in
 	    if (s > 0.0) then
@@ -499,12 +510,11 @@ try
 		set_timer s;
 		if (!coq) then coq_init();
 		read_thf_file (!problemfile) (find_read_thf_file (Filename.dirname (!problemfile)));
-		if (not (!slave)) then
-		  begin
-		    match (!coqchannel) with
-		    | Some c -> print_coqsig c
-		    | None -> ()
-		  end;
+		begin
+		  match (!coqchannel) with
+		  | Some c -> print_coqsig c
+		  | None -> ()
+		end;
 		search ()
 	      end
 	    else
@@ -522,15 +532,39 @@ try
 	  with
 	  | Unsatisfiable(r) ->
 	      begin
-		let c = match (!coqchannel) with Some c -> c | None -> stdout in
 		begin
-		  if (!mkproofterm) then begin match r with Some(r) -> print_proofterm c r | None -> () end;
-		  if (c != stdout) then
-		    close_out c;
+		  match (!mkproofterm,r) with
+		  | (Some CoqScript,Some r) ->
+		      let c = match (!coqchannel) with Some c -> c | None ->
+			begin
+			  print_string "% SZS output start Proof"; print_newline();
+			  print_string "% Coq Proof Script"; print_newline();
+			  stdout
+			end
+		      in
+		      begin
+			try print_coq_proofterm c r
+			with CoqProofTooBig coqproofsize ->
+			  begin
+			    if (!verbosity > 0) then Printf.printf "%% SZS status Success\nCoq Proof Too Big: %d steps\n" coqproofsize;
+			    exit 26;
+			  end
+		      end;
+		      if (c = stdout) then
+			begin
+			  print_string "% SZS output end Proof"; print_newline();
+			end
+		      else
+			close_out c;
+		  | (Some HOCore,Some r) ->
+		      print_string "% Higher-Order Unsat Core BEGIN"; print_newline();
+		      print_hocore_proofterm stdout r;
+		      print_string "% Higher-Order Unsat Core END"; print_newline();
+		  | _ -> ()
 		end;
 		match !conjecture with
-		| Some _ -> (if (!verbosity > 0) then Printf.printf "%% SZS status Theorem\n"; exit 20)
-		| None -> (if (!verbosity > 0) then Printf.printf "%% SZS status Unsatisfiable\n"; exit 25)
+		| Some _ -> (if (!verbosity > 0) then (Printf.printf "%% SZS status Theorem\n"; Printf.printf "%% %s \n" (String.concat " " (!mode))); exit 20)
+		| None -> (if (!verbosity > 0) then (Printf.printf "%% SZS status Unsatisfiable\n"; Printf.printf "%% %s \n" (String.concat " " (!mode))); exit 25)
 	      end
 	  | Satisfiable ->
 	      begin
