@@ -10,6 +10,7 @@ open Priorityqueue
 exception Redeclaration of string
 exception FileNotFound of string
 exception GenericError of string
+exception CoqProofTooBig of int
 
 let mode : string list ref = ref []
 let timeout : float option ref = ref None
@@ -17,8 +18,11 @@ let verbosity : int ref = ref 1 (*** Default to 1; 0 should mean print nothing (
 let coq = ref false
 let coqchannel : out_channel option ref = ref None
 let conjecturename : string ref = ref "claim"
-let conjecture : (pretrm * trm) option ref = ref None
-let mkproofterm = ref false
+let conjecture : (pretrm * trm * trm) option ref = ref None
+type proofkind = CoqScript | HOCore
+let mkproofterm = ref None
+let mkprooftermp () = match !mkproofterm with Some _ -> true | None -> false
+let slave1 = ref false
 let slave = ref false
 let coqsig_base : string list ref = ref []
 let coqsig_const : (string * stp) list ref = ref []
@@ -76,23 +80,7 @@ let get_atom m =
   try
     Hashtbl.find atom_hash m
   with
-  | Not_found ->
-      begin
-	if (get_bool_flag "SYM_EQ") then (*** Associate m1 = m2 and m2 = m1 with the same integer ***)
-	  begin
-	    match m with
-	    | Ap(Ap(Eq(b),m1),m2) ->
-		begin
-		  try
-		    Hashtbl.find atom_hash (Ap(Ap(Eq(b),m2),m1))
-		  with
-		  | Not_found -> new_atom m
-		end
-	    | _ -> new_atom m
-	  end
-	else
-	  new_atom m
-      end
+  | Not_found -> new_atom m
 
 let get_literal m =
   match (neg_body m) with
@@ -153,8 +141,8 @@ type ruleinfo =
   | ConfrontationRule of int * int
  (*** ChoiceRule(eps,pred) ***)
   | ChoiceRule of trm * trm
- (*** ClauseRule(nalllit,int list,trm list) - Universal quantifier, list of lits, list of instantiations to give the clause ***)
-  | ClauseRule of int * int list * trm list
+ (*** Known(lit,coqname,stp list) ***)
+  | Known of int * string * stp list
 
 type refut =
     NegImpR of trm * trm * refut
@@ -195,7 +183,7 @@ let ruleinfo_str s =
   | MatingRule(plit,nlit) -> "MatingRule(" ^ (string_of_int plit) ^ "," ^ (string_of_int nlit) ^ ")"
   | ConfrontationRule(plit,nlit) -> "ConfrontationRule(" ^ (string_of_int plit) ^ "," ^ (string_of_int nlit) ^ ")"
   | ChoiceRule(eps,pred) -> "ChoiceRule(" ^ (trm_str eps) ^ "," ^ (trm_str pred) ^ ")"
-  | ClauseRule(nalllit,lits,insts) -> "ClauseRule(" ^ (string_of_int nalllit) ^ "," ^ (String.concat ";" (List.map string_of_int lits)) ^ "," ^ (String.concat ";" (List.map trm_str insts)) ^ ")"
+  | Known(l,cname,al) -> "Known(" ^ (string_of_int l) ^ "," ^ cname ^ ")"
 
 (*** For printing the refutation - helpful for debugging. - Chad, Oct 2010 ***)
 let rec print_refut r =
@@ -523,11 +511,18 @@ let coq_init () =
     Hashtbl.add coq_used_names "False" ();
     Hashtbl.add coq_used_names "True" ();
     Hashtbl.add coq_used_names "not" ();
+    Hashtbl.add coq_used_names "Snot" ();
     Hashtbl.add coq_used_names "and" ();
+    Hashtbl.add coq_used_names "Sand" ();
     Hashtbl.add coq_used_names "or" ();
+    Hashtbl.add coq_used_names "Sor" ();
     Hashtbl.add coq_used_names "iff" ();
+    Hashtbl.add coq_used_names "Siff" ();
     Hashtbl.add coq_used_names "ex" ();
+    Hashtbl.add coq_used_names "SSigma" ();
+    Hashtbl.add coq_used_names "SPi" ();
     Hashtbl.add coq_used_names "eq" ();
+    Hashtbl.add coq_used_names "Seq" ();
     Hashtbl.add coq_used_names "I" ();
     Hashtbl.add coq_used_names "FalseE" ();
     Hashtbl.add coq_used_names "conj" ();
@@ -623,12 +618,13 @@ let print_coqsig c =
   begin
     Printf.fprintf c "Add LoadPath \"%s/coq\".\n" Config.satallaxdir;
     Printf.fprintf c "Require Import stttab.\n";
+    Printf.fprintf c "Section SatallaxProblem.\n";
     print_coqsig_base !coqsig_base;
     print_coqsig_const !coqsig_const;
     print_coqsig_def !coqsig_const;
     print_coqsig_hyp !coqsig_hyp;
     match (!conjecture) with
-    | Some (m,_) ->
+    | Some (m,_,_) ->
 	begin
 	  Printf.fprintf c "Theorem %s : " (Hashtbl.find coq_hyp_names (!conjecturename));
 	  print_pretrm_coq c m coq_names coq_used_names (-1) (-1);
@@ -797,11 +793,11 @@ let rec declare_thf_logic_formula (name:string) (role:string) (m:pretrm) =
 	      conjecturename := name;
 	      if (name != "claim") then
 		ignore (coqify_name name coq_hyp_names coq_used_names);
-	      conjecture := Some (m,tm);
 	      let ntm = Ap(Neg,tm) in
 	      let ntmn = norm name_def (logicnorm ntm) in
 	      initial_branch_prenorm := (ntm::(!initial_branch_prenorm));
 	      initial_branch := (ntmn::(!initial_branch));
+	      conjecture := Some (m,tm,ntmn);
 	      Hashtbl.add part_of_conjecture ntmn ()
 	    end
       end
