@@ -9,6 +9,7 @@ open Config
 exception Redeclaration of string
 exception FileNotFound of string
 exception GenericError of string
+exception CoqProofTooBig of int
 
 val mode : string list ref
 val timeout : float option ref
@@ -16,8 +17,11 @@ val verbosity : int ref
 val coq : bool ref
 val coqchannel : out_channel option ref
 val conjecturename : string ref
-val conjecture : (pretrm * trm) option ref
-val mkproofterm : bool ref
+val conjecture : (pretrm * trm * trm) option ref
+type proofkind = CoqScript | HOCore
+val mkproofterm : proofkind option ref
+val mkprooftermp : unit -> bool
+val slave1 : bool ref
 val slave : bool ref
 val coqsig_base : string list ref
 val coqsig_const : (string * stp) list ref
@@ -88,8 +92,8 @@ type ruleinfo =
   | ConfrontationRule of int * int
  (*** ChoiceRule(eps,pred) ***)
   | ChoiceRule of trm * trm
- (*** ClauseRule(nalllit,int list,trm list) - Universal quantifier, list of lits, list of instantiations to give the clause ***)
-  | ClauseRule of int * int list * trm list
+ (*** Known(lit,coqname,stp list) ***)
+  | Known of int * string * stp list
 
 val searchoption_str : searchoption -> string
 val ruleinfo_str : ruleinfo -> string
