@@ -262,24 +262,24 @@ let rec print_pretrm_coq c m h hu lp rp =
   | PAp(PForall,m1) ->
       if ((lp < 0) && (rp < 0)) then
 	begin
-	  Printf.fprintf c "!! ";
+	  Printf.fprintf c "SPi _ ";
 	  print_pretrm_coq c m1 h hu (-1) (-1);
 	end
       else
 	begin
-	  Printf.fprintf c "(!! ";
+	  Printf.fprintf c "(SPi _ ";
 	  print_pretrm_coq c m1 h hu (-1) (-1);
 	  Printf.fprintf c ")";
 	end
   | PAp(PExists,m1) ->
       if ((lp < 0) && (rp < 0)) then
 	begin
-	  Printf.fprintf c "?? ";
+	  Printf.fprintf c "SSigma _ ";
 	  print_pretrm_coq c m1 h hu (-1) (-1);
 	end
       else
 	begin
-	  Printf.fprintf c "(?? ";
+	  Printf.fprintf c "(SSigma _ ";
 	  print_pretrm_coq c m1 h hu (-1) (-1);
 	  Printf.fprintf c ")";
 	end
@@ -405,12 +405,16 @@ let rec print_pretrm_coq c m h hu lp rp =
       print_pretrm_coq c m2 h hu 19 (-1);
       Printf.fprintf c ")";
       if (rp >= 30) then Printf.fprintf c ")";
-  | PIff -> Printf.fprintf c "iff"
-  | POr -> Printf.fprintf c "or"
-  | PAnd -> Printf.fprintf c "and"
-  | PNeg -> Printf.fprintf c "not"
-  | PForall -> Printf.fprintf c "(fun p => forall x, p x)"
-  | PExists -> Printf.fprintf c "(fun p => exists x, p x)"
+  | PIff -> Printf.fprintf c "Siff"
+  | POr -> Printf.fprintf c "Sor"
+  | PAnd -> Printf.fprintf c "Sand"
+  | PNeg -> Printf.fprintf c "Snot"
+  | PForall ->
+      Printf.fprintf c "(SPi _)";
+  | PExists ->
+      Printf.fprintf c "(SSigma _)";
+  | PEq ->
+      Printf.fprintf c "(Seq _)";
   | PAp(m1,m2) ->
       if ((lp < 5000) && (rp < 5001)) then
 	begin
